from Tkinter import *

class EventArea(object):
  def __init__( self, x1, y1, x2, y2, evt):
    self.x1 = x1
    self.y1 = y1
    self.x2 = x2
    self.y2 = y2
    self.evt = evt

  def inside(self, x, y):
    if(x >= self.x1 and x <= self.x2 and y >= self.y1 and y <= self.y2):
      return True
    else:
      return False

 
class EventPlotter(Frame):
  def __init__( self
              , parent=None              
              , rowList=[]
              , eventList=[]
              , title='DEVS Plotter'
              , rowTitle='state'
              , plotterScale=1.0):
    Frame.__init__(self, parent)    
    #self.pack(expand=YES, fill=BOTH)
    self.parent         = parent
    self.rowList        = rowList
    self.eventList      = eventList
    self.title          = title
    self.rowTitle       = rowTitle
    self.scale          = plotterScale
    self.showTrace      = True
    self.showText       = False
    self.startNumber    = 0
    self.leftMargin     = 80
    self.rightMargin    = 40
    self.topMargin      = 30
    self.bottomMargin   = 30
    self.plotWidth      = 0
    self.plotHeight     = 0
    self.sourceY        = {}
    self.eventAreas     = [] 
    self.shownText      = []
    canv=Canvas(self, relief=SUNKEN) #, bg='brown')
    canv.config(width=300, height=200)
    #canv.config(highlightthickness=3) # no pixels to border    
    canv.pack(side=TOP, expand=YES, fill=BOTH)
    canv.bind('<Configure>', self.initPanel)
    #canv.bind('<Button-1>',  self.onLeftClick)            # mouse button clicks
    canv.bind('<Button-3>',  self.onRightClick)
    canv.bind('<Double-3>',  self.onDoubleRightClick)       # click left twice
    #canv.takefocus=1
    self.canvas = canv
    self.makeOperationArea()   
    
  def makeOperationArea(self):
    opArea = Frame(self, cursor='hand2', relief=RIDGE, bd=2)
    opArea.pack(side=BOTTOM, fill=X)        
  
    toolbar = Frame(opArea, cursor='hand2', relief=RIDGE, bd=2)
    toolbar.grid(row=0, column=1, sticky=NSEW)
    Button(toolbar, text='|<<', command=self.toBegin).grid(row=0, column=0)
    Button(toolbar, text='<<',  command=self.previousPage).grid(row=0, column=1)
    Button(toolbar, text='<',   command=self.previousOne).grid(row=0, column=2)
    Button(toolbar, text='>',   command=self.nextOne).grid(row=0, column=3)
    Button(toolbar, text='>>',  command=self.nextPage).grid(row=0, column=4)
    Button(toolbar, text='>>|', command=self.toEnd).grid(row=0, column=5)
    Button(toolbar, text='PostScript', command=self.SaveToPostScript).grid(row=0, column=6)
    Button(toolbar, text='Close', command=self.closeWindow).grid(row=0, column=7)
    optionbar = Frame(opArea, cursor='hand2', relief=RIDGE, bd=2)
    optionbar.grid(row=0, column=0, sticky=NSEW)
    l = Label(optionbar, text='Scale:')
    l.grid(row=0, column=0, sticky=E) #pack(side=LEFT)
    self.scaleEntry = Entry(optionbar, width=6)
    self.scaleEntry.insert(END, str(self.scale))
    self.scaleEntry.grid(row=0, column=1, sticky=W) #pack(side=LEFT)
    self.scaleEntry.bind('<KeyPress-Return>', self.setScale)
    
    #self.showTraceVar = IntVar()
    #ch=Checkbutton( optionbar
    #             , text='Line'
    #             , variable=self.showTraceVar
    #             , command=self.showTraceLine)
    #ch.select()
    #ch.grid(row=0, column=2) #pack(side=LEFT)

    self.showTextVar = IntVar()
    ch=Checkbutton( optionbar
                 , text='Text'
                 , variable=self.showTextVar
                 , command=self.showTraceText)
    ch.grid(row=0, column=2)
    
    opArea.rowconfigure(0, weight=1)
    opArea.columnconfigure(0, weight=1)
    opArea.columnconfigure(1, weight=1)
    optionbar.rowconfigure(0, weight=1)
    optionbar.columnconfigure(0, weight=1)
    optionbar.columnconfigure(1, weight=1)
    optionbar.columnconfigure(2, weight=2) 
    #optionbar.columnconfigure(3, weight=2)        
  
  #def showTraceLine(self):
  #  self.showTrace = self.showTraceVar.get()
  #  self.updateDisplay()

  def showTraceText(self):
    self.showText = self.showTextVar.get()
    self.updateDisplay()

  def onDoubleRightClick(self, event):
    if(len(self.shownText) > 0):
      for tArea in self.shownText:
        self.canvas.delete(tArea)
      self.shownText = []

  def onRightClick(self, event):
    if(len(self.eventAreas)>0):
      for area in self.eventAreas:
        if(area.inside(event.x, event.y)):
          txtContent = self.canvas.create_text( area.x2
                                              , area.y2+10
                                              , text=area.evt.getContent()
                                              , font=("Helvetica", "8")
                                              , anchor=NW)
          self.shownText.append(txtContent)
          return
    
#  def separateWindow(self):
#    win = Toplevel()
#    plotter = DevsPlotter( parent=win
#                         , rowList=self.rowList
#                         , eventList=self.eventList
#                         , title=self.title
#                         , rowTitle=self.rowTitle
#                         , plotterScale=self.scale)
#    plotter.pack(expand=YES, fill=BOTH)
#    plotter.mainloop()

  
  def setScale(self, event):
    scale = self.scaleEntry.get()
    if(len(scale)>0):
      self.scale = float(scale)
      self.updateDisplay()
  
#  def setTrace(self, trace):
#    self.showTrace = trace
    
  def initPanel(self, event):
    self.canvas.delete('all')
    row = len(self.rowList) + 1
    evtNumber   = len(self.eventList)
    rowInterval = (event.height-self.topMargin-self.bottomMargin)/row
    self.plotHeight = event.height-self.topMargin-self.bottomMargin
    self.plotWidth = event.width-self.leftMargin-self.rightMargin

    self.canvas.create_text( event.width/2
                           , self.topMargin/2
                           , text=self.title
                           , fill='blue'
                           , font=("Helvetica", "16"))

    for i in range(0, row):  
      if( i == row-1):
        self.canvas.create_line( self.leftMargin
                               , self.topMargin+rowInterval*(i+1) 
                               , self.leftMargin+self.plotWidth 
                               , self.topMargin+rowInterval*(i+1) 
                               , width=3 
                               , fill='blue' 
                               , arrow=LAST 
                               , arrowshape=(8,10,3))
        self.canvas.create_text( self.leftMargin+self.plotWidth 
                               , self.topMargin+rowInterval*(i+1)+5
                               , text='Time'
                               , anchor=N)
      else:
        self.canvas.create_line( self.leftMargin
                               , self.topMargin+rowInterval*(i+1)
                               , self.leftMargin+self.plotWidth 
                               , self.topMargin+rowInterval*(i+1)
                               , fill='grey')                               
        self.canvas.create_text( self.leftMargin/8
                               , self.topMargin+rowInterval*(i+1)
                               , width=self.leftMargin-4
                               , text=self.rowList[i]
                               , anchor=W
                               , fill='blue')
        self.sourceY[self.rowList[i]] = self.topMargin+rowInterval*(i+1)

    self.canvas.create_line( self.leftMargin
                           , self.topMargin
                           , self.leftMargin
                           , self.topMargin+rowInterval*row
                           , width=3
                           , fill='blue'
                           , arrow=FIRST
                           , arrowshape=(8,10,3))
    self.canvas.create_text( self.leftMargin/2
                           , self.topMargin
                           , text=self.rowTitle)
                           
    #if(len(self.eventList)>0):
    self.updateDisplay()
    
  def updateDisplay(self):

    if(len(self.shownText) > 0):
      for tArea in self.shownText:
        self.canvas.delete(tArea)
      self.shownText = []

    self.eventAreas = []

    if(len(self.eventList) > 0):
      tag = 'content'
      self.canvas.delete(tag)
      baseTime = self.eventList[self.startNumber].getTime()
      preEvt = None
      for evt in self.eventList[self.startNumber:]:
        #print evt 
        #print evt.state, self.startNumber
        if self.scale*(evt.getTime()-baseTime)< self.plotWidth:
          if(evt.getTime()>=0.0):
            ovalMark = self.canvas.create_oval( int(self.leftMargin+int(self.scale*(evt.getTime() - baseTime)))-4
                                              , self.sourceY[evt.getState()]-4
                                              , int(self.leftMargin+int(self.scale*(evt.getTime() - baseTime)))+4
                                              , self.sourceY[evt.getState()]+4
                                              , fill=''
                                              , outline="black")
            eArea = EventArea( int(self.leftMargin+int(self.scale*(evt.getTime() - baseTime)))-4
                             , self.sourceY[evt.getState()]-4
                             , int(self.leftMargin+int(self.scale*(evt.getTime() - baseTime)))+4
                             , self.sourceY[evt.getState()]+4
                             , evt)
            self.eventAreas.append(eArea)
            self.canvas.addtag_withtag(tag, ovalMark)                       

          if(self.showText):
            #if(preEvt):
            txtContent = self.canvas.create_text( self.leftMargin+int(self.scale*(evt.getTime() - baseTime))
                                                , self.sourceY[evt.getState()]  #self.sourceY[preEvt.state]
                                                , text=evt.getContent()
                                                , font=("Helvetica", "8"))
            self.canvas.addtag_withtag(tag, txtContent)                       
          txtTime = self.canvas.create_text( self.leftMargin+int(self.scale*(evt.getTime() - baseTime))
                                           , self.topMargin+self.plotHeight
                                           , text=("%7.2f" % evt.getTime())
                                           , anchor=N
                                           , font=("Helvetica", "8"))
          self.canvas.addtag_withtag(tag, txtTime)
          if( not (preEvt!=None and preEvt.getTime()==evt.getTime())):                                                                
            lineMark = self.canvas.create_line( self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                              , self.topMargin
                                              , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                              , self.topMargin + self.plotHeight - 3
                                              , fill='grey')                               
            self.canvas.addtag_withtag(tag, lineMark) 
          if(self.showTrace):
            if(preEvt):
              traceMark1 = self.canvas.create_line( self.leftMargin + int(self.scale*(preEvt.getTime() - baseTime))
                                                  , self.sourceY[preEvt.getState()]
                                                  , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                  , self.sourceY[preEvt.getState()]
                                                  , fill='black')
              self.canvas.addtag_withtag(tag, traceMark1) 
              endOvalMark = self.canvas.create_oval( int(self.leftMargin+int(self.scale*(evt.getTime() - baseTime)))-2
                                                   , self.sourceY[preEvt.getState()]-2
                                                   , int(self.leftMargin+int(self.scale*(evt.getTime() - baseTime)))+2
                                                   , self.sourceY[preEvt.getState()]+2
                                                   , fill='black')
              self.canvas.addtag_withtag(tag, endOvalMark)

              startOvalMark = self.canvas.create_oval( int(self.leftMargin+int(self.scale*(evt.getTime() - baseTime)))-4
                                                     , self.sourceY[evt.getState()]-4
                                                     , int(self.leftMargin+int(self.scale*(evt.getTime() - baseTime)))+4
                                                     , self.sourceY[evt.getState()]+4
                                                     , fill=''
																										 , outline='black')
              self.canvas.addtag_withtag(tag, startOvalMark)
                      
              if(evt.getType()=="IN"): #content.find("Int Trans")!=-1):
                if(self.sourceY[preEvt.getState()] > self.sourceY[evt.getState()]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))+10
                                                      , (self.sourceY[preEvt.getState()]+self.sourceY[evt.getState()])/2
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[evt.getState()]                                   
                                                      , fill='green'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2)                                     
                elif(self.sourceY[preEvt.getState()] < self.sourceY[evt.getState()]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))-10
                                                      , (self.sourceY[preEvt.getState()]+self.sourceY[evt.getState()])/2
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[evt.getState()]                                            
                                                      , fill='green'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2) 
                else: #equal
                  traceMark2 = self.canvas.create_line( self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))-10
                                                      , self.sourceY[preEvt.getState()]-10
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]-20
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))+10
                                                      , self.sourceY[preEvt.getState()]-10
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))+3
                                                      , self.sourceY[evt.getState()]-3                                     
                                                      , fill='green'
                                                      , arrow=LAST
                                                      , smooth=True)

                  self.canvas.addtag_withtag(tag, traceMark2)                                     

              elif(evt.getType()=="EX"): #content.find("Ext Trans")!=-1):
                if(self.sourceY[preEvt.getState()] > self.sourceY[evt.getState()]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))+10
                                                      , (self.sourceY[preEvt.getState()]+self.sourceY[evt.getState()])/2
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[evt.getState()]                                             
                                                      , fill='red'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2)                                     
                elif(self.sourceY[preEvt.getState()] < self.sourceY[evt.getState()]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))-10
                                                      , (self.sourceY[preEvt.getState()]+self.sourceY[evt.getState()])/2
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[evt.getState()]                                             
                                                      , fill='red'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2) 
                else: #equal
                  traceMark2 = self.canvas.create_line( self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))-10
                                                      , self.sourceY[preEvt.getState()]-10
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]-20
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))+10
                                                      , self.sourceY[preEvt.getState()]-10
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))+3
                                                      , self.sourceY[evt.getState()]-3                                     
                                                      , fill='red'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2)                                     

              else:
                if(self.sourceY[preEvt.getState()] > self.sourceY[evt.getState()]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))+10
                                                      , (self.sourceY[preEvt.getState()]+self.sourceY[evt.getState()])/2
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[evt.getState()]                                             
                                                      , fill='black'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2)                                     
                elif(self.sourceY[preEvt.getState()] < self.sourceY[evt.getState()]):
                  traceMark2 = self.canvas.create_line( self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))-10
                                                      , (self.sourceY[preEvt.getState()]+self.sourceY[evt.getState()])/2
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[evt.getState()]                                             
                                                      , fill='black'
                                                      , arrow=LAST
                                                      , smooth=True)                                                    
                  self.canvas.addtag_withtag(tag, traceMark2) 
                else: #equal
                  traceMark2 = self.canvas.create_line( self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))-10
                                                      , self.sourceY[preEvt.getState()]-10
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[preEvt.getState()]-20
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))+10
                                                      , self.sourceY[preEvt.getState()]-10
                                                      , self.leftMargin + int(self.scale*(evt.getTime() - baseTime))
                                                      , self.sourceY[evt.getState()]                                     
                                                      , fill='black'
                                                      , arrow=LAST
                                                      , smooth=True)
                  self.canvas.addtag_withtag(tag, traceMark2)                                     

            preEvt = evt
        else:
          break

  def newEvent(self, event): 
    self.eventList.append(event)
    if(self.scale*(event.getTime()-self.eventList[self.startNumber].getTime())>self.plotWidth):
      self.startNumber = self.startNumber + 1
    self.updateDisplay()
    
  def setEventSource(self, eventSource):
    eventSource.addEventListener(self)
    
  def previousOne(self):
    if(self.startNumber==0):
      return
    self.startNumber=self.startNumber-1    
    self.updateDisplay()
    
  def nextOne(self):
    if(self.startNumber==len(self.eventList)-1):
      return
    self.startNumber=self.startNumber+1
    self.updateDisplay()
    
  def toBegin(self):
    self.startNumber=0
    self.updateDisplay()
    
  def toEnd(self):
    self.startNumber=len(self.eventList)-1
    self.updateDisplay()

  def previousPage(self):
    if(self.startNumber==0):
      return    
    else:
      oldStart = self.startNumber
      self.startNumber -= 1
      xDistance = self.scale*(self.eventList[oldStart].getTime() - self.eventList[self.startNumber].getTime())
      while(self.startNumber>0 and xDistance < self.plotWidth):
        self.startNumber -= 1
        xDistance = self.scale*(self.eventList[oldStart].getTime() - self.eventList[self.startNumber].getTime())
    self.updateDisplay()
    
  def nextPage(self):
    maxNumber = len(self.eventList)-1
    if(self.startNumber==maxNumber):
      return    
    else:
      oldStart = self.startNumber
      self.startNumber += 1
      xDistance = self.scale*(self.eventList[self.startNumber].getTime() - self.eventList[oldStart].getTime())
      while(self.startNumber<maxNumber and xDistance < self.plotWidth):
        self.startNumber += 1
        xDistance = self.scale*(self.eventList[self.startNumber].getTime() - self.eventList[oldStart].getTime())
    self.updateDisplay()

  def SaveToPostScript(self):
    import tkFileDialog
    import os
    psfile = tkFileDialog.asksaveasfilename(initialdir = os.getcwd(), defaultextension = '.ps')
    self.canvas.postscript(file=psfile)

  def closeWindow(self):
    self.destroy()

if __name__ == '__main__':
  win = Tk()
  rowList = 'model0', 'model1', 'model2','model3','model4','model5'
  canv = EventPlotter(win, rowList)
  canv.pack(side=TOP)
  win.mainloop()
    

