from eventparser          import *
from Tkinter              import *                  # get widget classes
from tkMessageBox         import *                  # get standard dialogs
from tkFileDialog         import *                  # get askopenfiles dialog
from EventPlotter         import EventPlotter
import os
import sys
import string
#import pickle


class DEVSEnv(Frame):                                    # an extended frame
  def __init__(self, parent=None, modelDict=None):       # attach to top-level?
    Frame.__init__(self, parent)                         # do superclass init
    self.parent = parent
    self.pack(expand=YES, fill=BOTH)
    self.master.title("DEVS Simulation Trace Plotter")   # set window-manager info
    self.master.iconname("DEVS Plotter")                 # label when iconified
    self.toolBarFrame     = None
    self.stateFrame       = None
    self.traceParser      = None
    self.plottingModel    = None                         # model currently plotted
    self.mainPlotter      = None
    self.modelListbox     = None
    self.stateListbox     = None

    self.createWidgets()                                 # attach frames/widgets
        
  def createWidgets(self):
    self.makeMenuBar()
    self.makeToolBar()
      
    self.plotterFrame = Frame(self, cursor='hand2', relief=RIDGE, bd=2)
    self.plotterFrame.pack(expand=YES, fill=BOTH) #grid(row=1, column=1, sticky=NSEW)


  def makeToolBar(self):
    self.toolBarFrame = Frame(self, cursor='hand2', relief=SUNKEN, bd=2)
    self.toolBarFrame.pack(side=TOP, fill=X)

    #Build the list area
    listArea = Frame(self.toolBarFrame, cursor='hand2', relief=RAISED, bd=3)
    listArea.pack(side=LEFT, fill=Y)
    L = Label(listArea, text='Select a Model:')
    L.config(relief=FLAT, anchor=W)
    L.pack(side=TOP, fill=X)

    sbar = Scrollbar(listArea)
    listbox = Listbox(listArea, relief=SUNKEN)
    sbar.config(command=listbox.yview)
    listbox.config(yscrollcommand=sbar.set)
    listbox.config(selectmode=SINGLE, setgrid=1, width=30, height=5)
    listbox.bind('<Double-1>', self.modelListHandler)
    sbar.pack(side=RIGHT, fill=Y)
    listbox.pack(side=LEFT, fill=Y)
    self.modelListbox = listbox
        
  def makeMenuBar(self):
    self.menubar = Menu(self.master, relief=FLAT)
    self.master.config(menu=self.menubar)
    self.fileMenu()

  def fileMenu(self):
    pulldown = Menu(self.menubar)
    pulldown.add_command(label='Open...', command=self.openFile)
    pulldown.add_command(label='Quit',    command=self.quit)
    self.menubar.add_cascade(label='File', underline=0, menu=pulldown)
              
  def openFile(self):
    pathFile = askopenfilename( title       = "Select a trace file to open"
                              , filetypes   = [("Select an empty directory", "*.xml")]
			      , initialfile = ''
			      , initialdir   = os.getcwd()
			      , parent = self.parent
			      )
    if(pathFile):
      self.traceParser = TraceParser(pathFile)
      self.setModelList(self.traceParser.getModels())

  def setModelList(self, modelSet):
    if(self.modelListbox.size()>0):
      self.modelListbox.delete(0, self.modelListbox.size()-1)
    pos = 0
    for model in modelSet:
      self.modelListbox.insert(pos, model)
      pos = pos + 1

  def modelListHandler(self, event):
    index = self.modelListbox.curselection()
    label = self.modelListbox.get(index)
    self.plottingModel = label
    if(self.traceParser.getModelType(self.plottingModel)=='C'):
      self.plotCoupledModel()
    else:
      self.createStateList()

  def plotCoupledModel(self):
    self.mainPlotter = EventPlotter( parent       = self.plotterFrame
                                   , rowList      = self.traceParser.getSubComponents(self.plottingModel)
                                   , eventList    = self.traceParser.getModelEvents(self.plottingModel)
                                   , title        = self.plottingModel
                                   , rowTitle     = 'Sub Models'
                                   , plotterScale = 30
                                   )
    self.mainPlotter.pack(side=TOP, expand=YES, fill=BOTH, ipady=10)
    

  def createStateList(self):
  
    if(self.stateFrame):
      self.stateFrame.destroy()
      
    self.stateFrame = Frame(self.toolBarFrame, cursor='hand2', relief=RIDGE, bd=2)
    self.stateFrame.pack(side=LEFT) #grid(row=1, column=0, sticky=W)

    if(self.traceParser.getModelType(self.plottingModel)=='C'):
      txt = 'Sub-Models'
    else:
      txt = 'Plottable Attributes:'

    L=Label(self.stateFrame, text=txt)
    L.pack(side=TOP) #grid(row=0, column=0, sticky=NW)
    sbar = Scrollbar(self.stateFrame)
    listbox = Listbox(self.stateFrame, relief=SUNKEN)
    sbar.config(command=listbox.yview)
    listbox.config(yscrollcommand=sbar.set)
    listbox.config(selectmode=SINGLE, setgrid=1, width=40, height=5)
    listbox.bind('<Double-1>', self.stateListHandler)
    sbar.pack(side=RIGHT, fill=Y)
    listbox.pack(side=LEFT, fill=Y)

    pos = 0
    if(self.traceParser.getModelType(self.plottingModel)=='C'):
      state = self.traceParser.getSubComponents(self.plottingModel)
    else:
      state = self.traceParser.getModelStateAttributes(self.plottingModel)

    for attr in state:
      listbox.insert(pos, attr)
      pos = pos + 1

    self.stateListbox = listbox

  def stateListHandler(self, event):
    index = self.stateListbox.curselection()
    label = self.stateListbox.get(index)
    self.stateAttribute = label
    modelEvents = self.traceParser.getModelEvents(self.plottingModel)
    modelParser =  ModelParser(modelEvents, SimpleStateParser(self.stateAttribute))

    self.mainPlotter = EventPlotter( parent       = self.plotterFrame
                                   , rowList      = modelParser.getStates()
                                   , eventList    = modelParser.getEvents()
                                   , title        = self.plottingModel
                                   , rowTitle     = self.stateAttribute
                                   , plotterScale = 30
                                   )
    self.mainPlotter.pack(side=TOP, expand=YES, fill=BOTH, ipady=10)
  
  def quit(self):
    self.parent.destroy()
    
if __name__ == '__main__':
  root = Tk()
  devs = DEVSEnv(root)
  root.mainloop()  # if I'm run as a script
